function GCBackupManager()
{
	this.pref = Components.classes['@mozilla.org/preferences;1'].
	  						getService(Components.interfaces.nsIPrefBranch);
	this.localeBundle2 = document.getElementById("GmContBundle2");
	var date = new Date();
	this.backupDirName = date.getFullYear().toString()
					 					 + ("0"+(date.getMonth()+1)).substr(-2)
					 					 + ("0"+date.getDate()).substr(-2)
					 					 + "-"
					 					 + ("0"+date.getHours()).substr(-2)
					 					 + ("0"+date.getMinutes()).substr(-2)
					 					 + ("0"+date.getSeconds()).substr(-2)
					 					 + ("00"+date.getMilliseconds()).substr(-3);
}

GCBackupManager.prototype.getDumpDir = function(account)
{
	var file = gGCDirManager.getWorkingDir();
	if (file && account) file.append(account.replace("@","_"));
	return file;
}

GCBackupManager.prototype.getCacheDir = function()
{
	var file = this.getDumpDir();
	if (file) file.append("cache");
	
	return file;
}

GCBackupManager.prototype.getLast2Files = function(file)
{
	if (!file.exists()) return null;
	var list = new Array();
	var entries = file.directoryEntries;
	while (entries.hasMoreElements()) {
		var entry = entries.getNext().QueryInterface(Components.interfaces.nsILocalFile);
		if (!entry.isDirectory()) list.push(entry);
	}
	if (list.length == 0) return null;
	if (index != null && index >= list.length) return null;
	
	var sortFunc = function(a, b) {
		return a.leafName < b.leafName ? -1 : 1;
	};	
	
	list.sort(sortFunc);
	
	var ret = {
		first: list.pop(),
		second: list.pop()
	}
	
	return ret;
}

GCBackupManager.prototype.prepareDump = function(file, maxNum)
{
	if (!file.exists()) return;
	var list = new Array();
	var entries = file.directoryEntries;
	while (entries.hasMoreElements()) {
		var entry = entries.getNext().QueryInterface(Components.interfaces.nsILocalFile);
		//if (!entry.isDirectory()) list.push(entry);
		list.push(entry);
	}
	
	var sortFunc = function(a, b) {
		//return a.leafName < b.leafName ? -1 : 1;
		return a.lastModifiedTime < b.lastModifiedTime ? -1 : 1;
	};	
	
	list.sort(sortFunc);
	if (maxNum > -1) {
		var over = list.length - maxNum + 1;
		for (var i=0; i<over; i++) {
			if (list[i]) list[i].remove(true);
		}
	}
}

GCBackupManager.prototype.dump = function(file, account, str, ext, charset)
{
	file.append(this.backupDirName);
	var date = new Date();
	var name = account.replace("@","_")
					 + "-"
					 + date.getFullYear().toString()
					 + ("0"+(date.getMonth()+1)).substr(-2)
					 + ("0"+date.getDate()).substr(-2)
					 + "-"
					 + ("0"+date.getHours()).substr(-2)
					 + ("0"+date.getMinutes()).substr(-2)
					 + ("0"+date.getSeconds()).substr(-2)
					 + ("00"+date.getMilliseconds()).substr(-3)
					 + "." + ext;
	file.append(name);
	if (file.exists()) {
		file.remove(true);
	}
	file.create(file.NORMAL_FILE_TYPE, 0666);
	var ioService = Components.classes['@mozilla.org/network/io-service;1']
                    .getService(Components.interfaces.nsIIOService);

	var fileStream = Components.classes['@mozilla.org/network/file-output-stream;1']
										.createInstance(Components.interfaces.nsIFileOutputStream);
	fileStream.init(file, 2, 0x200, false);
	//use default charset
	//XXX: How about getting native encoding?
	var encoding = charset ? charset
												 : nsPreferences.copyUnicharPref(
												 									"gmcont.dump_charset",
												 									this.localeBundle2.getString("intl.charset.default")
												 									);
	//dump("Dump file: "+file.path+"\n");
	var converterStream;
	try {
		converterStream = Components
			.classes['@mozilla.org/intl/converter-output-stream;1']
			.createInstance(Components.interfaces.nsIConverterOutputStream);
		converterStream.init(fileStream, encoding, 0,
											 Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
		converterStream.writeString(str);
	} catch (e) {
		try {										 
			converterStream.init(fileStream, "UTF-8", 0,
												 Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
			converterStream.writeString(str);
		} catch(e) {
			gGCLogger.dump(e);
		}
	}
	converterStream.close();
	fileStream.close();
}

GCBackupManager.prototype.diff = function(file1, file2)
{
	if (!file1 || !file2) return true;
	
  var stream1 = Components.classes['@mozilla.org/network/file-input-stream;1'].createInstance(Components.interfaces.nsIFileInputStream);
  var stream2 = Components.classes['@mozilla.org/network/file-input-stream;1'].createInstance(Components.interfaces.nsIFileInputStream);
  try {
	  stream1.init(file1, 1, 0, false);
	  stream2.init(file2, 1, 0, false);
    var scriptableStream1 = Components.classes['@mozilla.org/scriptableinputstream;1'].createInstance(Components.interfaces.nsIScriptableInputStream);
    var scriptableStream2 = Components.classes['@mozilla.org/scriptableinputstream;1'].createInstance(Components.interfaces.nsIScriptableInputStream);
    scriptableStream1.init(stream1);
    scriptableStream2.init(stream2);
    var fileSize1 = scriptableStream1.available();
    var fileSize2 = scriptableStream2.available();
    
    if (fileSize1 != fileSize2) return true;
    
    while(scriptableStream1.available()) {
    	var fileContents1 = scriptableStream1.read(512);	
    	var fileContents2 = scriptableStream2.read(512);
    	if (fileContents1 != fileContents2) {
 		    scriptableStream1.close();
		    stream1.close();
 		    scriptableStream2.close();
		    stream2.close();
    		return true;
    	}
    }

    scriptableStream1.close();
    stream1.close();
    scriptableStream2.close();
    stream2.close();
    return false; // same files
  } catch(e) {
    return true;
  }	
}

GCBackupManager.prototype.isUpdateNeeded = function(account, feedXML, isGroup)
{
	var feedStr = feedXML.toString();
	feedStr = feedStr.replace(/gd:etag=\S+/,"");
	feedStr = feedStr.replace(/\<updated\>\S+\<\/updated\>/,"");
	
	var file = this.getDumpDir(account);
	file.append("xml");
	if (isGroup) file.append("groups");
	else file.append("contacts");

	this.prepareDump(file, 2);
	this.dump(file, account, feedStr, "xml", "UTF-8");
	
	file = this.getDumpDir(account);
	file.append("xml");
	if (isGroup) file.append("groups");
	else file.append("contacts");

	var diffFiles = this.getLast2Files(file);
	return diffFiles ? this.diff(diffFiles.first, diffFiles.second) : true;
	//return true;
}

GCBackupManager.prototype.dumpRawXMLString = function(account, feedStr, isGroup, startIndex)
{
	//var feedStr = feedXML.toString();
	//feedStr = feedStr.replace(/gd:etag=\S+/,"");
	//feedStr = feedStr.replace(/\<updated\>\S+\<\/updated\>/,"");
	
	var file = this.getDumpDir(account);
	if (!file) return;
	file.append("xml");
	if (isGroup) file.append("groups");
	else file.append("contacts");

	//this.prepareDump(file, 1);
	if (startIndex == 1) this.prepareDump(file, 1);
	else this.prepareDump(file, -1);
	this.dump(file, account, feedStr, "xml", "UTF-8");
}

GCBackupManager.prototype.copyTBAddressBook = function(account, abFile)
{
	var maxNum = nsPreferences.getIntPref("gmcont.max_backups", 0);
	if (maxNum <= 0) return;

	var file = this.getDumpDir(account);
	if (!file) return;
	
	file.append("Thunderbird");
	this.prepareDump(file, maxNum);	
	file.append(this.backupDirName);
	if (file.exists()) {
		file.remove(true);
	}
	/*
	file.append(name);
	if (file.exists()) {
		file.remove(true);
	}
	*/
	var name = abFile.leafName;
	abFile.copyTo(file, name);
}
